import { Component, Input, Output, EventEmitter, ChangeDetectionStrategy, Injector } from '@angular/core';
import { NavigationEvent } from './datepicker-view-model';
import { NgbDate } from './ngb-date';
import { NgbDatepickerI18nNOA11Y } from './datepicker-i18n';
import { NgbCalendarNOA11Y } from './ngb-calendar';
import { NgbDateStruct } from './ngb-date-struct';
import { NbpBaseComponent } from '@isp/xdce-widget/base';
import { CalendarItemNOA11Y } from './datepicker';


@Component({
  selector: 'ngb-fid-datepicker-navigation',
  changeDetection: ChangeDetectionStrategy.OnPush,
  host: { 'class': 'd-flex justify-content-between', '[class.collapsed]': '!showSelect' },
  templateUrl: './datepicker-navigation.html',
  styleUrls: ['./datepicker-navigation.scss'],
})
export class NgbDatepickerNavigationNOA11Y extends NbpBaseComponent{
  navigation = NavigationEvent;

  @Input() date: NgbDate;
  @Input() disabled: boolean;
  @Input() months: number;
  @Input() years: CalendarItemNOA11Y [][];
  @Input() showSelect: boolean;
  @Input() showWeekNumbers: boolean;
  @Input() template: number = 0;
  @Input() disabledWeekend: boolean;
  @Input() holidays: Array<NgbDateStruct>
  @Input() taxDeadlines: Array<NgbDateStruct>

  @Output() navigate = new EventEmitter<NavigationEvent>();
  @Output() select = new EventEmitter<NgbDate>();
  @Output() newTemplate = new EventEmitter<number>();

  _minDate: NgbDate;
  @Input()
  get minDate() {
    return this._minDate;
  }
  set minDate(minDate: NgbDate) {
    this._minDate = minDate;
  }

  _maxDate: NgbDate;
  @Input()
  get maxDate() {
    return this._maxDate;
  }
  set maxDate(maxDate: NgbDate) {
    this._maxDate = maxDate;
  }

  constructor(public i18n: NgbDatepickerI18nNOA11Y, private _calendar: NgbCalendarNOA11Y, private injector : Injector) {
    super(injector);
   }

  doNavigate(event: NavigationEvent) {
    this.navigate.emit(event);
  }

  nextDisabled() {
    let yearsSize = 0;
    this.years.forEach(row => yearsSize += row.length);
    return this.disabled || (this.maxDate && ((this.template == 0 && this._calendar.getNext(this.date, 'm').after(this.maxDate))
      || (this.template == 1 && this.date.year + 1 > this.maxDate.year)
      || (this.template == 2 && this.years[0][0].number + yearsSize > this.maxDate.year)));
  }

  prevDisabled() {
    let prevMonthDate = new Date(this.date.year, this.date.month + 1, 0);
    let prevMonth = new NgbDate(prevMonthDate.getFullYear(), prevMonthDate.getMonth() - 1, prevMonthDate.getDate());
    return this.disabled || (this.minDate && ((this.template == 0 && prevMonth.before(this.minDate))
      || (this.template == 1 && this.date.year - 1 < this.minDate.year)
      || (this.template == 2 && this.years[0][0].number - 1 < this.minDate.year)));
  }

  selectDate(date: NgbDate) {
    this.select.emit(date);
   }

  setTemplate() {
    this.template = (this.template + 1) % 3;
    this.newTemplate.emit(this.template);
  }

}
