import { Component, Input, Output, EventEmitter, ViewChild, ElementRef, OnInit } from '@angular/core';
import { NgbInputDatepickerNOA11Y } from './@ng-bootstrap-noa11y/datepicker/datepicker-input';
import { NbpCalendarPattern, NbpCalendarPosition, NbpDateSeparator } from './nbp-calendar-generic.enum';
import { NgbDateStruct } from './@ng-bootstrap-noa11y/datepicker/ngb-date-struct';
import { EventManager } from '@isp/xdce-arch-core-base';

@Component({
  selector: 'nbp-fid-calendar-generic',
  templateUrl: './nbp-fid-calendar-generic.component.html',
  styleUrls: ['./nbp-fid-calendar-generic.component.scss']
})
export class NbpFidCalendarGenericComponent implements OnInit {
  @ViewChild('datepickerInput') datepickerInput: NgbInputDatepickerNOA11Y;
  @ViewChild('inputElement') inputElement: ElementRef;

  @Input() placeholder: string = '';
  @Input() disabled: boolean = false;
  @Input() required: boolean = false;
  @Input() datePattern: NbpCalendarPattern = NbpCalendarPattern.GGMMAAAA;
  @Input() dateSeparator: NbpDateSeparator = NbpDateSeparator.SLASH;
  @Input() placementPosition: NbpCalendarPosition = NbpCalendarPosition.BOTTOM;
  @Input() minDate: NgbDateStruct;
  @Input() maxDate: NgbDateStruct;
  @Input() startDate: NgbDateStruct;
  @Input() showWeekNumbers: boolean = false;
  @Input() showWeekdays: boolean = true;
  @Input() showNavigation: boolean = true;
  @Input() outsideDays: 'visible' | 'collapsed' | 'hidden' = 'visible';
  @Input() displayMonths: number = 1;

  @Output() dateSelected = new EventEmitter<NgbDateStruct>();
  @Output() dateChanged = new EventEmitter<NgbDateStruct>();
  @Output() datepickerClosed = new EventEmitter<void>();

  constructor(private eventManager: EventManager) { }

  ngOnInit() {
    // Sottoscrizione all'evento di chiusura del datepicker
    this.eventManager.subscribeToEvent('DATEPICKER_CLOSE', () => {
      this.datepickerClosed.emit();
    });
  }

  /**
   * Apre il datepicker
   */
  open() {
    if (this.datepickerInput && !this.disabled) {
      this.datepickerInput.open(true);
    }
  }

  /**
   * Chiude il datepicker
   */
  close() {
    if (this.datepickerInput) {
      this.datepickerInput.close();
    }
  }

  /**
   * Alterna l'apertura e la chiusura del datepicker
   */
  toggle() {
    if (this.datepickerInput && !this.disabled) {
      this.datepickerInput.toggle(true);
    }
  }

  /**
   * Gestisce l'evento di selezione della data
   * @param date La data selezionata
   */
  onDateSelect(date: NgbDateStruct) {
    this.dateSelected.emit(date);
  }

  /**
   * Gestisce l'evento di modifica della data
   * @param date La nuova data
   */
  onDateChange(date: NgbDateStruct) {
    this.dateChanged.emit(date);
  }

  /**
   * Imposta il focus sull'input
   */
  focus() {
    if (this.inputElement) {
      this.inputElement.nativeElement.focus();
    }
  }
}